"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("../command");
const wasi_1 = require("@wasmer/wasi");
const node_1 = tslib_1.__importDefault(require("@wasmer/wasi/lib/bindings/node"));
const wasm_transformer_1 = require("@wasmer/wasm-transformer");
const blake3_1 = require("blake3");
const os = tslib_1.__importStar(require("os"));
const fs = tslib_1.__importStar(require("fs"));
const path = tslib_1.__importStar(require("path"));
const runWasiModule = async (flags) => {
    const args = flags._;
    if (args.length === 0 || args[0] === "help") {
        exports.runCommand.help();
        return;
    }
    const wasmModuleFileToRun = args[0];
    let dirFlag = [];
    if (flags.dir) {
        if (typeof flags.dir === "string") {
            dirFlag.push(flags.dir);
        }
        else {
            dirFlag = flags.dir;
        }
    }
    let mapdirFlag = [];
    if (flags.mapdir) {
        if (typeof flags.mapdir === "string") {
            mapdirFlag.push(flags.mapdir);
        }
        else {
            mapdirFlag = flags.mapdir;
        }
    }
    let envFlag = [];
    if (flags.env) {
        if (typeof flags.env === "string") {
            envFlag.push(flags.env);
        }
        else {
            envFlag = flags.env;
        }
    }
    const preopens = {};
    mapdirFlag.forEach((dir) => {
        const [wasm, host] = dir.split(":");
        if (!wasm || !host) {
            console.error("Error: Options to --mapdir= need to be in the format WASM_DIR:HOST_DIR");
            process.exit(1);
        }
        preopens[wasm] = host;
    });
    dirFlag.forEach((dir) => {
        preopens[dir] = dir;
    });
    const envVars = {};
    envFlag.forEach((env) => {
        const [key, value] = env.split(":");
        if (!key || !value) {
            console.error("Error: Options to ---env= need to be in the format KEY=VALUE");
            process.exit(1);
        }
        envVars[key] = value;
    });
    // Pass non-recognized / remaining args to the wasi module
    const recognizedArgs = ["--dir", "--mapdir", "--env", "--command-name", wasmModuleFileToRun];
    const wasiArgs = process.argv.slice(3).filter((arg) => {
        return !recognizedArgs.some(recognizedArg => arg.startsWith(recognizedArg) || arg === "--");
    });
    if (flags["command-name"]) {
        // Add provided command name in front of the args
        wasiArgs.unshift(flags["command-name"]);
    }
    else {
        // Add the wasm module to the front of the args.
        wasiArgs.unshift(wasmModuleFileToRun);
    }
    // console.log(preopens);
    const wasi = new wasi_1.WASI({
        args: wasiArgs,
        bindings: Object.assign(Object.assign({}, node_1.default), { fs }),
        env: envVars,
        preopens
    });
    let wasmBytes;
    try {
        wasmBytes = fs.readFileSync(wasmModuleFileToRun);
    }
    catch (e) {
        console.error(`Error: File ${wasmModuleFileToRun} not found.`);
        process.exit(1);
    }
    const tmpDir = os.tmpdir();
    const wasmerLoweredCacheDir = path.join(tmpDir, "wasmerjs-lowered-wasm");
    if (!fs.existsSync(wasmerLoweredCacheDir)) {
        fs.mkdirSync(wasmerLoweredCacheDir);
    }
    const hash = blake3_1.createHash();
    hash.update(wasmBytes);
    const digest = hash.digest().toString("hex");
    const loweredCachePath = path.join(wasmerLoweredCacheDir, digest);
    let transformedBytes;
    if (!fs.existsSync(loweredCachePath)) {
        transformedBytes = await wasm_transformer_1.lowerI64Imports(wasmBytes);
        // Save transformed bytes
        fs.writeFileSync(loweredCachePath, transformedBytes);
    }
    else {
        transformedBytes = fs.readFileSync(loweredCachePath);
    }
    const wasmModule = await WebAssembly.compile(transformedBytes);
    const instance = await WebAssembly.instantiate(wasmModule, Object.assign({}, wasi.getImports(wasmModule)));
    // const module = await WebAssembly.compile(transformedBytes);
    // const { instance } = await WebAssembly.instantiate(transformedBytes, {
    //   ...wasi.importsForModule(module)
    // });
    wasi.start(instance);
};
exports.runCommand = new command_1.Command({
    runCallback: runWasiModule,
    minimistConfig: {
        boolean: ["help", "version"],
        string: ["env", "dir", "mapdir", "command-name"],
        "--": true,
        alias: {
            help: ["h"],
            version: ["V"]
        }
    },
    description: "Run a WebAssembly file. Formats accepted: wasm, wat",
    getHelpBody: () => {
        return `USAGE:
    wasmer-js run [FLAGS] [OPTIONS] <path> [--] [--]...

FLAGS:
    -h, --help                              Prints help information
    -V, --version                           Prints version information

OPTIONS:
        --env <env-vars>...                  Pass custom environment variables
        --mapdir <mapped-dirs>...            Map a host directory to a different location for the wasm module
        --dir <pre-opened-directories>...    WASI pre-opened directory

ARGS:
    <path>     Input file
    <-->...    Application arguments`;
    }
});
